import re, math, pickle
from collections import Counter, defaultdict

import numpy as np
import pandas as pd
from scipy import stats

import matplotlib.pyplot as plt
plt.style.use('seaborn-whitegrid')
from matplotlib.pyplot import Polygon
from matplotlib import ticker
import seaborn as sns
sns.set(style='whitegrid')

# fixed_color_list = ["#E64B35B2", "#4DBBD5B2", "#00A087B2", "#3C5488B2", "#F39B7FB2", "#8491B4B2",
# "#91D1C2B2", "#DC0000B2", "#7E6148B2"]
# chr_color_list = list(colors._colors_full_map.values())

def insertion_distribution(data, output_root):
  max_x, max_y = max(data['insertion_length']), max(data['length_count'])
  fig, ax = plt.subplots(figsize=(8, 8))
  # 散点
  ax.scatter( data['insertion_length'], data['length_count'], color='black', alpha=0.6 )
  # 阴影 用polygon
  poly1=Polygon([(0, 0),(0, max_y), (20, max_y), (20, 0)],alpha=0.3, facecolor='#FFE6D0', edgecolor="black", linestyle="-." )
  poly2=Polygon([(20, 0), (20, max_y), (30, max_y), (30, 0)],alpha=0.3, facecolor='#FFFDE5', edgecolor="black", linestyle="-."  )
  ax.add_patch(poly1)
  ax.add_patch(poly2)
  # text
  plt.text(max_x *0.5, 1, 'degradation rate:' + str(round(data['degradation_ratio'], 2) ), zorder=99 )
  plt.text(10, max_y *0.1, 'high degradation', ha="left", va="bottom", rotation=90, zorder=99)
  plt.text(25, max_y *0.1, 'partial degradation', ha="left", va="bottom", rotation=90, zorder=99)
  # ax.text(0.7, 0.9, 'degradation ratio ' + str(round(degradation_ratio, 2) ) , ha='center', va='center', transform=ax.transAxes, zorder=99, backgroundcolor='white')
  plt.xlim([0, max_x*1.05])
  plt.xlabel('Size of insertion')
  plt.ylabel('Number of reads')
  # ax.set_yscale('log')
  # plt.tight_layout()
  formatter = ticker.ScalarFormatter(useMathText=True)
  formatter.set_scientific(True)
  formatter.set_powerlimits((-1,1))
  ax.yaxis.set_major_formatter(formatter)

  plt.savefig(output_root + 'imgs/adapter_insertion_distribution.png')
  plt.savefig(output_root + 'imgs/adapter_insertion_distribution.pdf' )
  plt.close(fig)

def readsLength_distribution(reads_length_data, output_root ): # table_data, table_col, table_row
  fig, axes = plt.subplots(len(reads_length_data['steps']), 1, sharex=True, sharey=True, figsize=(14, 20))

  for i, step in enumerate(reads_length_data['steps']):
    x, y = reads_length_data['bar_list'][i]
    axes[i].bar(x, y)
    axes[i].set_ylabel('' )
    axes[i].grid( axis='y')
    axes[i].set_title('Preprocess: ' + step, fontsize = 14, fontweight ='bold', verticalalignment='bottom', loc ='left')
    axes[i].text(0.01, 0.9, reads_length_data['annotate_list'][i],
        horizontalalignment='left',
        verticalalignment='top',
        transform=axes[i].transAxes)
    axes[i].spines['top'].set_visible(False)
    axes[i].spines['right'].set_visible(False)
    axes[i].spines['bottom'].set_visible(False)
    # axes[i].spines['left'].set_visible(False)
  plt.xlabel('Reads length')
  plt.yscale("log")
  sns.despine(left=True)
  plt.savefig(output_root + 'imgs/reads_distribution.png')
  plt.savefig(output_root + 'imgs/reads_distribution.pdf')
  plt.close(fig)

def q20_q30_line( data, output_root):
  fig, ax = plt.subplots(figsize=(8, 8))
  x = [1, 2, 3, 4, 5]
  l1, l2 = ax.plot(x, data['q20_list'], x, data['q30_list'] , lw=2)
  l1.set_label('Q20')
  l2.set_label('Q30')
  ax.legend()
  ax.set_xticks([1, 2, 3, 4, 5])
  ax.set_xticklabels(data['steps'], fontsize=18) #tex code
  plt.xticks(rotation=30)

  plt.tight_layout()
  plt.savefig(output_root + 'imgs/quality_trend.png')
  plt.savefig(output_root + 'imgs/quality_trend.pdf')
  plt.close(fig)

def preprocess_summary_stack(data, output_root ):
  x_have, x_not, x_failed = data['stackbar_list']
  fig, ax = plt.subplots(figsize=(8,8))
  plt.bar(data['steps'], x_have, label="reads with feature",edgecolor = 'black')
  plt.bar(data['steps'], x_not, label="reads without feature",edgecolor = 'black', bottom = x_have)
  plt.bar(data['steps'], x_failed, label="failed reads",edgecolor = 'black', bottom = [i+j for i, j in zip(x_have,x_not)])
  plt.legend( bbox_to_anchor=(0.5, 1.2), loc='upper center', ncol=3, fontsize='large' )
  plt.xticks(rotation=30)
  plt.ylabel('Reads number',fontsize=12)

  for x1, y1, y2, y3 in zip(data['steps'], x_have, x_not, x_failed):
    p1 = y1/(y1+y2+y3)
    p2 = y2/(y1+y2+y3)
    p3 = y3/(y1+y2+y3)
    # print( p1, p2, p3)
    if p1 >0.05:
      plt.text(x1, y1 * 0.8, '{:.0%}'.format(p1), ha='center', va="top")
    if p2>0.05:
      plt.text(x1, y1 + (y2)* 0.4,  '{:.0%}'.format(p2), ha='center')
    if p3>0.05:
      plt.text(x1, y1 + y2 + (y3)* 0.2, '{:.0%}'.format(p3), ha='center', va="bottom")

  # plt.yscale('log')
  formatter = ticker.ScalarFormatter(useMathText=True)
  formatter.set_scientific(True)
  formatter.set_powerlimits((-1,1))
  ax.yaxis.set_major_formatter(formatter)
  plt.tight_layout()
  plt.savefig(output_root +'imgs/reads_ratio.png')
  plt.savefig(output_root +'imgs/reads_ratio.pdf')
  plt.close(fig)

def mapping_ratio_stack(data, output_root):
  x_un, x_uni, x_mul = data['map_ratio_list']
  fig = plt.figure(figsize=(8,8)) #设置画布的尺寸
  plt.bar(data['soft'], x_un, label="unmapped reads",edgecolor = 'black')
  plt.bar(data['soft'], x_uni, label="unique mapped reads",edgecolor = 'black', bottom = x_un)
  plt.bar(data['soft'], x_mul, label="non-unique mapped reads",edgecolor = 'black', bottom = [i+j for i, j in zip(x_un,x_uni)])
  # plt.legend( loc=3,fontsize=14)  # 设置图例位置
  plt.legend( bbox_to_anchor=(0.5, 1.2), loc='upper center', ncol=3, fontsize='large' )
  plt.ylabel('Mapping reads number',fontsize=16)
  plt.xlabel('Mapping reads split method',fontsize=16)

  for x1, y1, y2, y3 in zip(data['soft'], x_un, x_uni, x_mul):
    p1 = y1/(y1+y2+y3)
    p2 = y2/(y1+y2+y3)
    p3 = y3/(y1+y2+y3)
    if p1 >0.05:
      plt.text(x1, y1 * 0.4, '{:.0%}'.format(p1), ha='center',fontsize = 15)
    if p2>0.05:
      plt.text(x1, y1 + (y2)* 0.4,  '{:.0%}'.format(p2), ha='center',fontsize = 15)
    if p3>0.05:
      plt.text(x1, y1 + y2 + (y3)* 0.4, '{:.0%}'.format(p3), ha='center',fontsize = 15)

  plt.tight_layout()
  plt.savefig(output_root +'imgs/mapping_split.png')
  plt.savefig(output_root +'imgs/mapping_split.pdf')
  plt.close(fig)

def exon_intron_scatter( data, output_root ):
  fig, ax = plt.subplots(figsize=(8,8))
  sns.regplot( data=data['filter_exon_intron_df1'], x='exon', y='intron', marker="x",
  scatter_kws = {'alpha' : 0.2 },
  line_kws={"color": 'black'}, x_jitter = 0.5, y_jitter = 0.5,  ax=ax, truncate = True )
  plt.text(0.2, 0.75, data['text'], transform = ax.transAxes)
  # plt.title( 'Exon/Intron ratio',  )
  plt.xlabel('Exon density')
  plt.ylabel('Intron density')
  plt.xlim([0, None])
  plt.ylim([0, None])

  fig.savefig(output_root + 'imgs/exon_intron_ratio.png')
  fig.savefig(output_root + 'imgs/exon_intron_ratio.pdf')
  fig.tight_layout()
  plt.close(fig)

def chr_rpkm_boxplot( data, output_root ):
  fig, ax = plt.subplots(figsize=(8, 8))
  bp = ax.boxplot(data['rpkm_list'], patch_artist=True) #, boxprops={'facecolor': fixed_color_list[0], 'color':fixed_color_list[0] })

  # for box in bp['boxes']:
  #   box.set(facecolor='#087E8B', alpha=0.6, linewidth=2)

  for whisker in bp['whiskers']:
    whisker.set(linewidth=2)

  for median in bp['medians']:
    median.set(color='black', linewidth=3)

  # ax.set_title( data['title'] )
  # ax.set_xlabel( x_label)
  ax.set_ylabel( 'log10(RPKM)' )

  ax.set_xticklabels( [x.replace('chr', '') for x in data['sort_chr_list']], fontsize=7)
  fig.savefig(output_root + 'imgs/chr_rpkm.png')
  fig.savefig(output_root + 'imgs/chr_rpkm.pdf')
  # plt.yscale('log')
  plt.tight_layout()
  plt.close(fig)

def expressed_pie(data, output_root):
  fig = plt.figure(figsize=(12, 12))
  # bigger
  bigger = plt.pie(data['big_sizes'], labels=data['big_labels'], labeldistance=0.75, startangle=90, frame=True, textprops={'fontsize': 12, 'color': 'w'})
  # [autotext.set_color('white') for autotext in autotexts]
  '''
  smaller = plt.pie(sizes_vegefruit, labels=labels_vegefruit,
                    colors=colors_vegefruit, radius=0.7,
                    startangle=90, labeldistance=0.7)
  '''

  wedges, texts = plt.pie(data['small_sizes'], wedgeprops=dict(width=0.5),
    radius=0.7, labeldistance=0.7,  startangle=90)
  bbox_props = dict(boxstyle="square,pad=0.3", fc="white", ec="k", lw=0.72)
  kw = dict(arrowprops=dict(arrowstyle="-", color='black'),
    bbox=bbox_props, zorder=100, va="center")

  for i, p in enumerate(wedges):
    ang = (p.theta2 - p.theta1)/2. + p.theta1
    y = np.sin(np.deg2rad(ang))
    x = np.cos(np.deg2rad(ang))

    horizontalalignment = {-1: "right", 1: "left"}[int(np.sign(x))]
    connectionstyle = "angle,angleA=0,angleB={}".format(ang)
    kw["arrowprops"].update({"connectionstyle": connectionstyle})
    plt.annotate(data['small_labels'][i],
    xy=(x*0.6, y*0.6), xytext=(0.8*np.sign(x), 0.8*y),
      horizontalalignment=horizontalalignment, **kw)

  centre_circle = plt.Circle((0, 0), 0.4, color='white', linewidth=0)
  # fig = plt.gcf()
  fig.gca().add_artist(centre_circle)

  plt.axis('equal')
  plt.tight_layout()
  plt.savefig(output_root + 'imgs/expressed_genes.png')
  plt.savefig(output_root + 'imgs/expressed_genes.pdf')
  plt.close(fig)

def expressed_distribution(data, output_root):
  protein_data, linc_data = data['protein_data'], data['linc_data']
  protein_data1 = np.log10( protein_data[protein_data >0] )
  linc_data1 = np.log10( linc_data[linc_data >0] )
  protein_density = stats.gaussian_kde( protein_data1 )
  linc_density = stats.gaussian_kde( linc_data1 )
  # print( min(protein_data1), max(protein_data1), min(linc_data1), max(linc_data1) )
  protein_x = np.linspace(min(protein_data1), max(protein_data1) )
  linc_x = np.linspace(min(linc_data1), max(linc_data1) )

  fig, ax = plt.subplots(figsize=(8,8))
  plt.plot(protein_x,protein_density(protein_x), label='Coding')
  plt.plot(linc_x,linc_density(linc_x), label='LncRNA')
  ax.set_xlabel('log10(RPKM)')
  ax.set_ylabel('density')
  ax.legend(loc='best')
  plt.tight_layout()
  plt.savefig(output_root + 'imgs/expression_distribution.png')
  plt.savefig(output_root + 'imgs/expression_distribution.pdf')
  plt.close(fig)

def proximal_pausing_sites_distribution(site_index_list, output_root):
  fig, ax = plt.subplots(figsize=(8,8))
  ax = sns.histplot(data=pd.DataFrame({'pause site': site_index_list}), x='pause site', kde=True, ax=ax,  alpha=0.75)
  plt.xlabel('Pause site')
  plt.ylabel('Frequency')
  # plt.text(325, int(np.max(site_index_list)*0.9), s='Mean=%s\nMedian=%s'%(int(np.mean(site_index_list)), int(np.median(site_index_list))), verticalalignment="top",horizontalalignment="right")
  ax.annotate('Mean=%s\nMedian=%s'%(int(np.mean(site_index_list)), int(np.median(site_index_list)) ), xy=(0.05, 0.85), xycoords='axes fraction')
  sns.despine(fig=None, ax=None, top=False, right=False, left=False, bottom=False, offset=None, trim=False)
  plt.savefig(output_root+'imgs/proximal_pause_site.png')
  plt.savefig(output_root+'imgs/proximal_pause_site.pdf')
  plt.close(fig)

def global_distribution_lollipopplot(data, output_root):
  fig, axes = plt.subplots(len(data['chrs'])*2, 1, figsize=(  30, 2*len(data['chrs']) ),  sharex=True, sharey=False)
  plt.xticks([])

  i = 0
  for chr in data['chrs']:
    # color = chr_color_list[i]
    try:
      axes[i].stem( data['lollipop_dic'][chr]['forward']['site'], data['lollipop_dic'][chr]['forward']['count'], markerfmt='.' ) #, linefmt=color,  basefmt=color  )
    except:
      pass
    axes[i].set_ylabel( chr+':+') # , color=color )
    axes[i].yaxis.tick_right()
    axes[i].yaxis.get_major_ticks()[1].set_visible(False)
    axes[i].yaxis.get_major_ticks()[-2].set_visible(False)
    axes[i].spines['top'].set_visible(False)
    axes[i].spines['left'].set_visible(False)
    # axes[i].spines['right'].set_color( color )
    i+=1
    try:
      axes[i].stem( data['lollipop_dic'][chr]['reverse']['site'], [-1*x for x in data['lollipop_dic'][chr]['reverse']['count']], markerfmt='.') #, linefmt=color,  basefmt=color )
    except:
      pass
    axes[i].set_ylabel( chr+':-') # , color=color)
    axes[i].yaxis.tick_right()
    axes[i].yaxis.get_major_ticks()[1].set_visible(False)
    axes[i].yaxis.get_major_ticks()[-2].set_visible(False)
    axes[i].spines['bottom'].set_visible(False)
    axes[i].spines['left'].set_visible(False)
    # axes[i].spines['right'].set_color(color)
    i+=1

  # plt.subplots_adjust(left=0.1, right=0.9, top=0.9, bottom=0.1)
  # plt.title('Genome wide pausing sites distribution', y=1.1, fontsize=18)
  fig.tight_layout()
  fig.savefig(output_root + 'imgs/pause_sites.png', bbox_inches='tight')
  fig.savefig(output_root + 'imgs/pause_sites.pdf', bbox_inches='tight')
  # plt.show()
  plt.close(fig)


def in_out_degree(data, output_root):
  plt.figure(figsize=(8, 8))
  plt.loglog(range(len(data['in_degree'])), data['in_degree'], marker='o', label='in-degree')
  plt.loglog(range(len(data['out_degree'])), data['out_degree'], marker='o', label='out-degree')
  plt.xlabel('Degree')
  plt.ylabel('Frequency')
  plt.legend()
  plt.savefig(output_root + 'imgs/network_degree.png')
  plt.savefig(output_root + 'imgs/network_degree.pdf')
  plt.close()





import os, sys, pickle, fire
script_dir = os.path.abspath(os.path.join(os.path.abspath(__file__), ".."))
lib_dir = os.path.abspath(os.path.join(os.path.abspath(__file__), '../../libs') )
sys.path.append(lib_dir)
from collections import defaultdict, Counter

import numpy as np

import matplotlib.pyplot as plt
plt.style.use('seaborn-whitegrid')
from matplotlib.pyplot import Polygon
from matplotlib import ticker
import seaborn as sns
sns.set(style='whitegrid')

import networkx as nx
import hvplot.networkx as hvnx
from community import community_louvain

from py_ext import dic2csv, two_list_2_dict
from plot import in_out_degree

def network_motif(data, output_root):
  fig = plt.figure(figsize=(16, 9))
  axgrid = fig.add_gridspec(5, 14)

  ax0 = fig.add_subplot(axgrid[0, 0], ylabel='Triads' )
  ax0.set_ylabel('motif', fontdict={'size': 16}, rotation=0)

  # ax0.set_axis_off()
  ax0.spines['right'].set_visible(False)
  ax0.spines['top'].set_visible(False)
  ax0.spines['bottom'].set_visible(False)
  ax0.spines['left'].set_visible(False)
  ax0.set_xticks([])
  ax0.set_yticks([])

  triad_list = [
    "021D", "021U", "021C", "111D", "111U", "030T", "030C",
    "201", "120D", "120U", "120C", "210", "300"
  ]
  triad_graph_list = [nx.triad_graph( triad ) for triad in triad_list]
  for i, graph in enumerate(triad_graph_list):
      tmp_ax = fig.add_subplot(axgrid[0, i+1] )
      nx.draw_spectral(graph, ax =tmp_ax, node_size=12 )
      plt.title( i+1 )

  ax1 = fig.add_subplot(axgrid[1:5, :])
  #ax0.grid(True, which='minor')
  #ax0.axhline(y=0, color='k')
  plt.xticks(np.arange(min(data['xAxis']), max(data['xAxis'])+1, 1.0))
  ax1.bar(data['xAxis'], data['triad_count_list']  )
  ax1.set_ylabel('Quantity', fontdict={'size': 16})
  ax1.set_xlabel('Motif', fontdict={'size': 16})
  ax1.set_yscale('log')
  plt.tight_layout()
  plt.savefig(output_root + 'imgs/network_motif.png')
  plt.savefig(output_root + 'imgs/network_motif.pdf')
  plt.close()



def get_source_target(source_file):
  source_set, target_set, source_target_edge_set = set(), set(), set()
  for ln in open(source_file):
    # if ln.startswith('hsa-'): continue
    ls = ln.split(',')
    source, target =ls[0], ls[1].strip()
    source_set.add(source)
    target_set.add(target)
    source_target_edge_set.add( ( source, target ) )

  target_set = target_set - source_set
  return  list( source_set ), list( target_set ), list(source_target_edge_set)

def generate_graph(tf_nodes, protein_nodes, edges):
  G = nx.DiGraph()
  G.add_nodes_from( tf_nodes, shape = 'D')
  G.add_nodes_from( protein_nodes, shape='o' )
  G.add_edges_from( edges )
  return G

def filter_nodes(G, attrs_dic, over_condition=0):
  filter_nodes = []
  for node, attr in attrs_dic.items():
    try:
      float(attr)
    except:
      continue
    if float(attr) > over_condition:
      filter_nodes.append(node)
  sub_G = G.subgraph(filter_nodes)
  return sub_G

def network_summary_info(G, output_root, tf_num, enhancer_num):
  graph_info = {}
  graph_info['nodes_num'] = nx.number_of_nodes(G)
  graph_info['edges_num'] = nx.number_of_edges(G)
  graph_info['tf_num'] = tf_num
  graph_info['enhancer_num'] = enhancer_num
  graph_info['mean_degree'] = 2* graph_info['edges_num']/graph_info['nodes_num']
  try:
    graph_info['assortativity_coefficient'] = nx.degree_assortativity_coefficient(G) # 匹配系数
  except Exception as e:
    print( e )
  try:
    graph_info['correlation_coefficient'] = nx.degree_pearson_correlation_coefficient(G) # 相关性系数
  except Exception as e:
    print( e )
  # graph_info['closeness_centrality'] = nx.closeness_centrality(G) # 节点距离中心系数
  # graph_info['betweenness_centrality'] = nx.betweenness_centrality(G) # 节点介数中心系数
  try:
    graph_info['transitivity'] = nx.transitivity(G) # 图或网络的传递性。
  except Exception as e:
    print( e )
  # graph_info['clustering'] = nx.clustering(G) # 图或网络中节点的聚类系数,用来度量连接的密度。
  # graph_info['average_clustering'] = nx.average_clustering(G) # nx.clustering(G) 的均值
  # graph_info['square_clustering'] = nx.square_clustering(G)
  try:
    graph_info['density'] = nx.density(G)
  except Exception as e:
    print( e )
  dic2csv(graph_info, output_root + 'csv/graph_info.csv')


def degree_analysis(G, output_root):
  def degree_histogram_directed(G, in_degree=False, out_degree=False):
    """Return a list of the frequency of each degree value.

    Parameters
    ----------
    G : Networkx graph
        A graph
    in_degree : bool
    out_degree : bool

    Returns
    -------
    hist : list
        A list of frequencies of degrees.
        The degree values are the index in the list.

    Notes
    -----
    Note: the bins are width one, hence len(list) can be large
    (Order(number_of_edges))
    """
    nodes = G.nodes()
    if in_degree:
      in_degree = dict(G.in_degree())
      degseq=[in_degree.get(k,0) for k in nodes]
    elif out_degree:
      out_degree = dict(G.out_degree())
      degseq=[out_degree.get(k,0) for k in nodes]
    else:
      degseq=[v for k, v in G.degree()]
    dmax=max(degseq)+1
    freq= [ 0 for d in range(dmax) ]
    for d in degseq:
      freq[d] += 1
    return freq

  in_degree_freq = degree_histogram_directed(G, in_degree=True)
  out_degree_freq = degree_histogram_directed(G, out_degree=True)
  # degrees = range(len(in_degree_freq))
  in_out_degree_dic = {'in_degree': in_degree_freq, 'out_degree': out_degree_freq}
  in_out_degree( in_out_degree_dic, output_root)
  # with open('in_out_degree_dic.pickle', 'wb') as f:
  #   pickle.dump(in_out_degree_dic, f)


def computeTriads(graph):
    triads_16 = nx.triadic_census(graph)
    triads_13 = {x: y for x,y in triads_16.items() if(x != '003' and x != '012' and x != '102')}
    return triads_13

def motif_analysis( G, output_root ):
  outdeg = G.out_degree()
  # 这步要 缩减一下几点 否则 triadic 跑不动
  to_keep = [n[0] for n in outdeg if n[1] > 3]
  g = G.subgraph(to_keep)

  # triads_16 = nx.triadic_census( g )
  triads_13 = computeTriads(g)

  xAxis = [1,2,3,4,5,6,7,8,9,10,11,12,13]
  triad_list = [
    "021D", "021U", "021C", "111D", "111U", "030T", "030C",
    "201", "120D", "120U", "120C", "210", "300"
  ]
  triad_count_list =[triads_13[t] for t in triad_list]

  motif_dic = {'xAxis': xAxis, 'triad_count_list': triad_count_list}
  network_motif(motif_dic, output_root)
  with open('motif_dic.pickle', 'wb') as f:
    pickle.dump(motif_dic, f)



def community_analysis( G, node_type_dic, output_root ):
  type_list = list( set( node_type_dic.values() ) )
  shape_list = 'od^ps>v<h8'
  if len(type_list) > len( shape_list):
    print('Error, node type is over', len(shape_list) )
  type_shape_dic = two_list_2_dict( type_list, shape_list[: len(type_list)] )
  partition = community_louvain.best_partition( nx.to_undirected( G ),resolution=1)
  # dic2json(partition, output_root + 'json/partition.json' )

  community_dic = defaultdict( lambda: [])
  for n, c in partition.items():
    community_dic[c].append(n)

  dic2csv({'community_' + str(c): len(n_list) for c, n_list in community_dic.items()}, output_root + 'csv/community_node_num.csv')
  for c, n_list in community_dic.items():
    with open( output_root + 'txt/community_%s.txt'%c, 'w' ) as f:
      for n in n_list:
        f.write(n + '\n')

    g = G.subgraph( n_list )
    keep_nodes = [n for n,d in g.degree if d >= 2 ]
    g = g.subgraph( keep_nodes )
    if len(g) <=10:
      print( 'community %s nodes less than 10'%str(c) )
      continue
    if len( g ) > 1000:
      print( 'community %s nodes more than 1000'%str(c) )
      continue

    pos = nx.spring_layout(g, k=0.4, iterations=10)
    type_nodes_dic = {type: [] for type in type_list}
    type_pos_dic = {type: {} for type in type_list}
    for n in g:
      try:
        type = node_type_dic[n]
        type_nodes_dic[type].append(n)
        type_pos_dic[type][n] = pos[n]
        # node_communitity_dic[n] = c
      except:
        continue
    fig = plt.figure(figsize=(12,12))
    # nx.draw( g, pos, node_shape=[type_shape_dic[node_type_dic[n] ] for n in g] )

    hv_final = hvnx.draw_networkx_edges(g, pos, connectionstyle="arc3,rad=0.1", width=950, height=950, edge_line_width=0.1 )

    for type in type_nodes_dic.keys():
      # if 10 < len( g ) < 400:
      hv_final = hv_final * hvnx.draw_networkx_nodes(g.subgraph(type_nodes_dic[type]), type_pos_dic[type], node_shape=type_shape_dic[type], node_size = [g.degree[node] * 10 for node in type_nodes_dic[type]], label=type  )

      nx.draw_networkx_nodes(g, pos, node_shape=type_shape_dic[type], nodelist = [x for x in g.subgraph(type_nodes_dic[type])], label='%s'%type )

    hvnx.save( hv_final, output_root + 'html/community_' +str(c)+'.html')

    nx.draw_networkx_edges(g,pos)
    plt.legend(scatterpoints = 1)
    plt.savefig(output_root + "imgs/community_%s.png"%str(c), format="PNG")
    plt.close(fig)

def main(tf_source=None, enhancer_source=None, select_nodes_file=None, express_file=None, output_root='./tmp_output/' ):
  # 节点 + 边
  total_source, total_target, total_edges =[], [], []
  # 节点 属性
  node_type_dic, node_express_dic = {}, {}


  # 1 使用source file 获取 节点+边
  tf_list, enhancer_list = [], []
  if tf_source:
    tf_list, gene_list, tf_target_edge_list = get_source_target(tf_source)
    print( 'tf gene number is %s, target gene number is %s'%( str(len(tf_list) ), str(len(gene_list)) ))
    total_source.extend( list( set(tf_list) ) )
    total_target.extend( list( set(gene_list) ) )
    total_edges.extend( tf_target_edge_list )

    # 节点属性
    for gene in gene_list:
      node_type_dic[gene] = 'gene'
    for tf in tf_list:
      node_type_dic[tf] = 'tf'

  if enhancer_source:
    enhancer_list, gene_list, enhancer_target_edge_list = get_source_target(enhancer_source)
    print( 'enhancer gene number is %s, target gene number is %s'%( str(len(enhancer_list) ), str(len(gene_list)) ))
    total_source.extend( list( set(enhancer_list) ) )
    total_target.extend( list( set(gene_list) ) )
    total_edges.extend( enhancer_target_edge_list )
    # 节点属性
    for gene in gene_list:
      if gene not in node_type_dic.keys():
        node_type_dic[gene] = 'gene'
    for enhancer in enhancer_list:
      node_type_dic[enhancer] = 'enhancer'


  # filter_nodes用于筛选的节点 包含了 tf, enhancer, gene。 最后的网络只在filter_nodes中
  filtered_sources, filtered_targets = total_source, total_target
  if select_nodes_file:
    select_node_set = set([ln.strip() for ln in open(select_nodes_file)])
    filtered_sources = list( select_node_set & set(filtered_sources) )
    filtered_targets = list( select_node_set & set(filtered_targets) )

  # print( len(filtered_targets), len(filtered_sources))
  if express_file:
    express_node_set = set()
    for ln in open(express_file):
      try:
        gene, count = ln.strip().split(',')
        if float(count) > 0:
          express_node_set.add( gene )
          node_express_dic[gene] = float(count)
      except:
        pass
    # print( len(express_node_set) )
    filtered_sources = list( set(filtered_sources) & express_node_set)
    filtered_targets = list( set(filtered_targets) & express_node_set)

  print( 'after filtering, source gene number is %s, target gene number is %s'%( str(len(filtered_sources) ), str(len(filtered_targets) )) )

  filter_edges = []
  for edge in total_edges:
    s, t = edge[0], edge[1]
    if s in filtered_sources and t in filtered_targets:
      filter_edges.append(edge)

  G = generate_graph(filtered_targets, filtered_sources, filter_edges)
  network_summary_info(G, output_root, len( set(tf_list) & set(filtered_sources) ), len( set(enhancer_list) & set(filtered_sources) ) )
  # attr_dic = {ln.split(',')[0]: ln.split(',')[1] for ln in open(filter_nodes)}
  # print( len(protein_list), len(tf_list), len(tf_protein_edge_set))
  # G = generate_graph(protein_list, tf_list, tf_protein_edge_set)
  # filter_G = filter_nodes(G, attr_dic)
  # 2 degree 分析
  degree_analysis(G, output_root)

  # 3 motif 分析
  motif_analysis(G, output_root)

  # 4 社区发现
  community_analysis( G, node_type_dic, output_root )

if __name__ == '__main__':
  fire.Fire(main)


import os, fire, psutil

class BatchNasap(object):
  def _run_batch_cmd(self, para_dic, cmd_list):
    filter_para_dic = {k:v for k, v in para_dic.items() if v}
    for k, v in filter_para_dic.items():
      if k == 'self': continue
      if k == 'read_file': continue
      cmd_list.extend(['--'+k, str(v)])
    os.system( ' '.join(cmd_list))
    # print( ' '.join(cmd_list) )

  def all(self, read_file=None, bowtie_index=None,  gtf=None, output_root=None, cores=1, adapter1=None, adapter2=None):
    all_para_dic = locals()

    for read_ln in open(read_file):
      para_dic = all_para_dic.copy()
      if ',' in read_ln:
        para_dic['read1'] = read_ln.split(',')[0].strip()
        para_dic['read2'] = read_ln.split(',')[1].strip()
      else:
        para_dic['read1'] = read_ln.strip()
      self._run_batch_cmd(para_dic , ['nasap', 'all'] )

def main():
  fire.Fire(BatchNasap)

if __name__ == '__main__':
  main()


import sys, os
from glob import glob
from setuptools import setup,find_packages

snap_files = []
snapdir = './nasap/scripts/snap/**'

snap_directories = [f for f in glob(snapdir, recursive=True) if os.path.isdir( f )]
# print( snap_directories )
for directory in snap_directories:
  snap_files.append((directory, [os.path.join(directory, f) for f in os.listdir(directory) if not os.path.isdir(os.path.join(directory, f))]) )

gencore_files = ['nasap/scripts/gencore/gencore']
bg2bw_files = ['nasap/scripts/ucsc/bedGraphToBigWig']

data_files = [('nasap/scripts/', glob('nasap/scripts/[!_]*.py', recursive=True) ),
  ('nasap/scripts/', glob('nasap/scripts/[!_]*.bash', recursive=True) ),
  ('nasap/scripts/templates/', ['nasap/scripts/templates/template.html', 'nasap/scripts/templates/template_track.txt']),
  ('nasap/scripts/templates/static/', ['nasap/scripts/templates/static/vue.min.js']),
]
# print( snap_files )
data_files.extend(snap_files)
data_files.extend( gencore_files )
data_files.extend( bg2bw_files )

def get_version():
  with open( os.path.abspath(os.path.dirname(__file__) ) + '/nasap/__version__.py') as f:
    ver = f.read().split('version=')[1].replace('"', '').strip()
  return ver

def main():
  root = os.path.abspath(os.path.dirname(__file__))
  with open(os.path.join(root, 'requirements.txt')) as f:
    install_requires = f.read().splitlines()
  # linux depandance packages
  # linux_deps = ['fastp', 'bioawk', 'python', 'bowtie2', 'samtools', 'bedtools', 'deeptools']
  # for dep in linux_deps:
  #   is_dep = os.system(dep + ' --version')
  #   if is_dep != 0:
  #     apt install

  # if float(sys.version[:3])< 3.6:
  #   sys.stderr.write("CRITICAL: Python version must be >= 3.6x!\n")
  #   sys.exit(1)

  setup(
    name='nasap',
    version=get_version(),
    description='This is nASAP setup file',
    author='biodancer',
    author_email='szxszx@foxmail.com',
    url='https://github.com/biodancerwanghzi/nasap/',
    # packages= find_packages(exclude=["back"]),
    packages=['nasap'],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    data_files= data_files,
    entry_points={
      'console_scripts': [
        'nasap=nasap.nasap:main',
        'batch_nasap=nasap.batch_nasap:main'
      ]
    },
    install_requires=install_requires,
    extras_require={
      'genomeBrowser': [
        'pyGenomeTrack'
    ]

    },
    # zip_safe = True
  )

if __name__ == '__main__':
  main()


import os, sys
sys.path.append(os.path.dirname(os.path.realpath(__file__))) # https://stackoverflow.com/questions/16981921/relative-imports-in-python-3
sys.path.append(os.path.dirname(os.path.realpath(__file__+'/libs')))
sys.path.append(os.path.dirname(os.path.realpath(__file__+'/scripts')))